package com.simuwang.daq.components.report.parser;

import cn.hutool.core.collection.CollUtil;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dos.EmailFieldMappingDO;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.smppw.common.pojo.ValueLabelVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;

public abstract class AbstractReportParser<T extends ReportData> implements ReportParser<T> {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final EmailFieldMappingMapper fieldMappingMapper;
    /**
     * 字段匹配规则
     */
    protected List<ValueLabelVO> fieldMapper;

    public AbstractReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        this.fieldMappingMapper = fieldMappingMapper;
    }

    @Override
    public void init() {
        List<EmailFieldMappingDO> emailFieldMapping = this.fieldMappingMapper.getEmailFieldMapping();
        if (CollUtil.isEmpty(emailFieldMapping)) {
            this.logger.error("未设置报告解析规则！");
            return;
        }
        this.fieldMapper = emailFieldMapping.stream().map(e -> new ValueLabelVO(e.getCode(), e.getName())).collect(Collectors.toList());
    }
}
