package com.simuwang.daq.components.report.parser.pdf;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.Constants;
import com.simuwang.base.common.exception.APIException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.*;
import com.simuwang.daq.components.report.parser.ReportParserConstant;
import com.smppw.common.pojo.ValueLabelVO;
import org.springframework.stereotype.Component;
import technology.tabula.RectangularTextContainer;
import technology.tabula.Table;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2024/9/11 16:19
 * @description pdf格式的月报解析
 */
@Component(ReportParserConstant.PARSER_PDF_MONTHLY)
public class PDMonthlyReportParser extends AbstractPDReportParser<MonthlyReportData> {
    private final List<Table> extNavTables = ListUtil.list(true);
    private Table fundInfoTable = null;

    public PDMonthlyReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public String getParser() {
        return ReportParserConstant.PARSER_PDF_MONTHLY;
    }

    @Override
    protected void initTableInfo(List<Table> tables) {
        for (Table table : tables) {
            int colCount = table.getColCount();
            if (colCount == 4) {
                this.fundInfoTable = table;
            } else if (colCount >= 5) {
                this.extNavTables.add(table);
            }
        }
    }

    @Override
    protected ReportFundInfoDTO parseFundInfo(ReportParserParams params) {
        Table fundInfoTable = this.fundInfoTable;
        if (fundInfoTable == null) {
            throw new APIException("未解析到基本信息表格");
        }
        Map<String, Object> baseInfoMap = MapUtil.newHashMap(32);
        for (int i = 0; i < fundInfoTable.getRows().size(); i++) {
            List<RectangularTextContainer> cols = fundInfoTable.getRows().get(i);
            for (int j = 0; j < 2; j++) {
                baseInfoMap.put(cols.get(j * 2).getText(), cols.get(j * 2 + 1).getText());
            }
        }
        // 匹配字段清洗字段
        ReportFundInfoDTO reportFundInfo = new ReportFundInfoDTO();
        this.buildInfo(baseInfoMap, reportFundInfo);
        return reportFundInfo;
    }

    @Override
    protected MonthlyReportData parseExtInfoAndSetData(String reportName, ReportFundInfoDTO fundInfo, ReportParserParams params) {
        MonthlyReportData reportData = new MonthlyReportData();
        reportData.setBaseInfo(this.buildReportInfo(params.getFileId(), reportName));
        reportData.setFundInfo(fundInfo);

        List<ReportNetReportDTO> exts = ListUtil.list(false);
        List<Table> extNavTables = this.extNavTables;
        for (Table extNavTable : extNavTables) {
            Map<String, Object> extInfoMap = MapUtil.newHashMap(16);
            for (int i = 0; i < extNavTable.getColCount(); i++) {
                String key = extNavTable.getCell(0, i).getText();
                String value = extNavTable.getCell(1, i).getText();
                extInfoMap.put(key, value);
            }
            ReportNetReportDTO navInfo = new ReportNetReportDTO();
            buildInfo(extInfoMap, navInfo);
            exts.add(navInfo);
        }
        reportData.setNetReport(exts);
        return reportData;
    }

    private void buildInfo(Map<String, Object> extInfoMap, Object info) {
        for (Map.Entry<String, Object> entry : extInfoMap.entrySet()) {
            String k = entry.getKey();
            Object v = entry.getValue();
            String fieldValue = StrUtil.toStringOrNull(v);
            if (fieldValue.startsWith("-") || fieldValue.endsWith("-")) {
                fieldValue = null;
            }
            if (fieldValue != null) {
                fieldValue = fieldValue.replace("\r", Constants.EMPTY);
            }
            for (ValueLabelVO vo : this.fieldMapper) {
                String fieldName = vo.getValue();
                List<String> labels = StrUtil.split(vo.getLabel(), ",");
                if (labels.contains(k)) {
                    try {
                        ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                    } catch (Exception e) {
                        this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                    }
                    break;
                }
                for (String label : labels) {
                    if (k.contains(label)) {
                        try {
                            ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                        } catch (Exception e) {
                            this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                        }
                        break;
                    }
                }
            }
        }
    }
}
